#!/usr/bin/env python
'''Migrates a Bob package recipe from bob.conda into its own repository

Usage: {prog} <template_recipe> <old_recipe> <conda_build_variant> <package_folder>
'''

from __future__ import absolute_import, division, print_function
import re
from conda_build.api import render, Config
from conda_build.variants import parse_config_file
from conda_build.conda_interface import PY3
import os

RUN_EXPORTS = [
    '    - bzip2\n',
    '    - dbus\n',
    '    - expat\n',
    '    - ffmpeg\n',
    '    - fontconfig\n',
    '    - freetype\n',
    '    - giflib\n',
    '    - glib\n',
    '    - gmp\n',
    '    - gst-plugins-base\n',
    '    - gstreamer\n',
    '    - hdf5\n',
    '    - icu\n',
    '    - jpeg\n',
    '    - kaldi\n',
    '    - libblitz\n',
    '    - libboost\n',
    '    - libffi\n',
    '    - libmatio\n',
    '    - libogg\n',
    '    - libopus\n',
    '    - libpng\n',
    '    - libsvm\n',
    '    - libtiff\n',
    '    - libvpx\n',
    '    - libxcb\n',
    '    - libxml2\n',
    '    - menpo\n',
    '    - mkl-devel\n',
    '    - mkl\n',
    '    - ncurses\n',
    '    - openfst\n',
    '    - openssl\n',
    '    - readline\n',
    '    - sox\n',
    '    - speex\n',
    '    - speexdsp\n',
    '    - sqlite\n',
    '    - tk\n',
    '    - vlfeat\n',
    '    - xz\n',
    '    - yaml\n',
    '    - zlib\n',
]

RUN_EXPORTS += [
    '    - bob.buildout\n',
    '    - bob.extension\n',
    '    - bob.blitz\n',
    '    - bob.core\n',
    '    - bob.io.base\n',
    '    - bob.sp\n',
    '    - bob.ap\n',
    '    - bob.math\n',
    '    - bob.measure\n',
    '    - bob.db.base\n',
    '    - bob.io.audio\n',
    '    - bob.io.image\n',
    '    - bob.io.video\n',
    '    - bob.io.matlab\n',
    '    - bob.ip.base\n',
    '    - bob.ip.color\n',
    '    - bob.ip.draw\n',
    '    - bob.ip.gabor\n',
    '    - bob.learn.activation\n',
    '    - bob.learn.libsvm\n',
    '    - bob.learn.linear\n',
    '    - bob.learn.mlp\n',
    '    - bob.learn.em\n',
    '    - bob.learn.boosting\n',
    '    - bob.db.iris\n',
    '    - bob.db.wine\n',
    '    - bob.db.mnist\n',
    '    - bob.db.atnt\n',
    '    - bob.ip.facedetect\n',
    '    - bob.ip.optflow.hornschunck\n',
    '    - bob.ip.optflow.liu\n',
    '    - bob.ip.flandmark\n',
    '    - bob.bio.base\n',
    '    - bob.bio.caffe_face\n',
    '    - bob.bio.face\n',
    '    - bob.bio.gmm\n',
    '    - bob.bio.spear\n',
    '    - bob.bio.video\n',
    '    - bob.db.arface\n',
    '    - bob.db.asvspoof\n',
    '    - bob.db.asvspoof2017\n',
    '    - bob.db.atvskeystroke\n',
    '    - bob.db.avspoof\n',
    '    - bob.db.banca\n',
    '    - bob.db.biosecure\n',
    '    - bob.db.biosecurid.face\n',
    '    - bob.db.casme2\n',
    '    - bob.db.caspeal\n',
    '    - bob.db.cohface\n',
    '    - bob.db.cuhk_cufs\n',
    '    - bob.db.frgc\n',
    '    - bob.db.fv3d\n',
    '    - bob.db.gbu\n',
    '    - bob.db.hci_tagging\n',
    '    - bob.db.ijba\n',
    '    - bob.db.kboc16\n',
    '    - bob.db.lfw\n',
    '    - bob.db.livdet2013\n',
    '    - bob.db.mobio\n',
    '    - bob.db.msu_mfsd_mod\n',
    '    - bob.db.multipie\n',
    '    - bob.db.nist_sre12\n',
    '    - bob.db.pericrosseye\n',
    '    - bob.db.putvein\n',
    '    - bob.db.replay\n',
    '    - bob.db.replaymobile\n',
    '    - bob.db.scface\n',
    '    - bob.db.utfvp\n',
    '    - bob.db.verafinger\n',
    '    - bob.db.voicepa\n',
    '    - bob.db.voxforge\n',
    '    - bob.db.xm2vts\n',
    '    - bob.db.youtube\n',
    '    - bob.ip.caffe_extractor\n',
    '    - bob.ip.facelandmarks\n',
    '    - bob.ip.skincolorfilter\n',
    '    - bob.kaldi\n',
    '    - bob.pad.base\n',
    '    - bob.pad.face\n',
    '    - bob.pad.voice\n',
    '    - bob.rppg.base\n',
    '    - gridtk\n',
]

BUILD_ONLY_DEPS = [
    '    - pkg-config {{ pkg_config }}\n',
    '    - cmake {{ cmake }}\n',
]


def extract_requirements_text(path):
  text = ""
  if path:
    with open(path) as f:
      recipe_text = f.read()
    if PY3 and hasattr(recipe_text, 'decode'):
      recipe_text = recipe_text.decode()
    match = re.search(
        r'(^requirements:.*?)(^test:|^extra:|^about:|^outputs:|\Z)',
        recipe_text, flags=re.MULTILINE | re.DOTALL)
    text = match.group(1) if match else ""
  return text


def extract_test_text(path):
  text = ""
  if path:
    with open(path) as f:
      recipe_text = f.read()
    if PY3 and hasattr(recipe_text, 'decode'):
      recipe_text = recipe_text.decode()
    match = re.search(
        r'(^test:.*?)(^extra:|^about:|^outputs:|\Z)',
        recipe_text, flags=re.MULTILINE | re.DOTALL)
    text = match.group(1) if match else ""
  i1 = text.find('requires:\n') + 10
  i2 = text.find('imports:', i1)
  i3 = text.find('commands:', i1)
  i2 = i2 if i2 > -1 else len(text)
  i3 = i3 if i3 > -1 else len(text)
  i2 = min(i2, i3)
  return text[i1:i2]


def add_variant(doc, star=True):
  lines = doc.split('\n')
  for i, line in enumerate(lines):
    if line.find('{{') == -1:
      if line.find(' - ') == -1:
        continue
      variant = line.split('-', 1)[1].strip()
      if variant.startswith('bob.') or variant == 'bob':
        continue
      variant = variant.replace('-', '_').replace('.', '_')
      variant = ' {{ ' + variant.split('#')[0].strip() + ' }}'
      line = line.split('#')
      line = [line[0] + variant] + line[1:]
      lines[i] = '#'.join(line)
  doc = '\n'.join(lines)
  if star:
    doc = doc.replace('}}.*', '}}').replace('}}', '}}.*')
    doc = doc.replace("compiler('c') }}.*", "compiler('c') }}")
    doc = doc.replace("compiler('cxx') }}.*", "compiler('cxx') }}")
  return doc


def extract_section(text, first, last):
  i1 = text.find(first)
  i2 = text.find(last, i1)
  if i1 == -1 or i2 == -1:
    return -1, -1, ''
  return i1, i2, text[i1 + len(first):i2]


def migrate(template_recipe_path, old_recipe_path, build_variant_path,
            output_folder):
  config = Config()
  variants = parse_config_file(build_variant_path, config)
  recipe = render(old_recipe_path, variants=variants)[0][0]
  with open(template_recipe_path) as f:
    final_recipe = f.read()
  final_recipe = final_recipe.replace('<PACKAGE>', recipe.name())
  entry_points, entry_points_test = '', ''
  if 'entry_points' in recipe.meta['build']:
    entry_points = '  entry_points:\n    - ' + \
        '\n    - '.join(recipe.meta['build']['entry_points']) + '\n'
    entry_points_test = '    - ' + '\n    - '.join(
        l.split('=')[0].strip() + ' --help' for l in
        recipe.meta['build']['entry_points']) + '\n'
  final_recipe = final_recipe.replace('  <ENTRY_POINTS>\n', entry_points)
  final_recipe = final_recipe.replace(
      '    <ENTRY_POINTS_TEST>\n', entry_points_test)
  requirements_text = extract_requirements_text(old_recipe_path)
  i1, i2, build_text = extract_section(
      requirements_text, 'build:\n', 'host:\n')
  i1, i2, host_text = extract_section(requirements_text, 'host:\n', 'run:\n')
  already_host = [
      '- python\n',
      '- setuptools\n',
      '- python {{ python }}\n',
      '- setuptools {{ setuptools }}\n',
      '- toolchain {{ toolchain }}\n',
      "- {{ compiler('c') }}\n",
      "- {{ compiler('cxx') }}\n",
  ]
  for line in already_host:
    host_text = host_text.replace(line, '')
  host_text = host_text.strip()
  if host_text:
    host_text = '    ' + host_text + '\n'
    host_text = add_variant(host_text, star=False)
  # normalize build_text
  build_text = build_text.rstrip()
  if build_text:
    build_text = '  build:\n' + build_text + '\n'
  # remove BUILD_ONLY_DEPS from host and put it in build_text
  for p in BUILD_ONLY_DEPS:
    if p in host_text:
      host_text = host_text.replace(p, '')
      build_text += p
  final_recipe = final_recipe.replace('    <HOST_DEPS>\n', host_text)
  final_recipe = final_recipe.replace('  <BUILD_DEPS>\n', build_text)

  run_text = requirements_text[i2 + 5:]
  for line in already_host[:2] + RUN_EXPORTS:
    run_text = run_text.replace(line, '')
  run_text = run_text.strip()
  run_text = '    ' + run_text + '\n' if run_text else run_text
  final_recipe = final_recipe.replace('    <RUN_DEPS>\n', run_text)

  already_test = ['nose', 'coverage', 'sphinx', 'sphinx_rtd_theme']
  already_test += [l + ' {{ ' + l + ' }}' for l in already_test]
  already_test = ['    - ' + l + '\n' for l in already_test]
  test_requires = extract_test_text(old_recipe_path)
  for line in already_test:
    test_requires = test_requires.replace(line, '')
  test_requires = test_requires.strip()
  if test_requires:
    test_requires = '    ' + test_requires + '\n'
    if recipe.name() == 'bob.buildout':
      test_requires = add_variant(test_requires)
  if recipe.name() == 'bob.buildout':
    final_recipe = final_recipe.replace(
        '    - bob-devel {{ bob_devel }}.*\n', '')
  final_recipe = final_recipe.replace(
      '    <TEST_DEPS>\n', test_requires)
  final_recipe = final_recipe.replace(
      '<LICENSE>', recipe.meta['about']['license'])
  final_recipe = final_recipe.replace(
      '<SHORT_DESCRIPTION>', recipe.meta['about']['summary'])
  license_family = recipe.meta['about']['license']
  license_family = 'BSD' if 'BSD' in license_family else 'GPL'
  final_recipe = final_recipe.replace('<LICENSE_FAMILY>', license_family)

  output_folder = os.path.join(output_folder, 'conda')
  os.makedirs(output_folder, exist_ok=True)
  out_path = os.path.join(output_folder, 'meta.yaml')

  # custom modifications:
  if recipe.name() == 'bob.extension':
    final_recipe = final_recipe.replace(
        '-sv {{ name }}', '-sv {{ name }} --exclude=test_extensions')

  if recipe.name() in ('bob.ip.gabor', 'bob.learn.em'):
    final_recipe = final_recipe.replace(
        '{{ sphinx_rtd_theme }}\n', '{{ sphinx_rtd_theme }}\n' +
        '    - matplotlib {{ matplotlib }}\n')

  with open(out_path, 'w') as f:
    f.write(final_recipe)

  # fix the doc/conf.py
  latex_conf = '''import sphinx
if sphinx.__version__ >= "1.4.1":
    extensions.append('sphinx.ext.imgmath')
    imgmath_image_format = 'svg'
else:
    extensions.append('sphinx.ext.pngmath')

'''
  place_to_insert = '''    'sphinx.ext.viewcode',
'''
  replace = place_to_insert + place_to_insert.replace('viewcode', 'mathjax')
  conf = os.path.join(output_folder, '..', 'doc', 'conf.py')
  with open(conf) as f:
    lines = f.read()
  lines = lines.replace(latex_conf, '')
  lines = lines.replace(place_to_insert, replace)
  with open(conf, 'w') as f:
    f.write(lines)


def main(argv=None):
  import sys
  migrate(*sys.argv[1:])


if __name__ == '__main__':
  main()
